#include frex:shaders/api/player.glsl
#include frex:shaders/api/view.glsl
#include frex:shaders/api/world.glsl

out float v_diffuse;
out float v_skyDiffuse;
out float v_heldCosIn;
out float v_heldCosOut;

#ifdef SHADOW_MAP_PRESENT
out vec4 v_shadowViewPos;
#endif

void frx_pipelineVertex()
{
  if (frx_isGui) {
    // Position of hand and GUI items
    gl_Position = frx_guiViewProjectionMatrix * frx_vertex;
  } else {
    // Position of world objects
    frx_vertex += frx_modelToCamera;
    gl_Position = frx_viewProjectionMatrix * frx_vertex;
  }

  if (frx_isHand) {
      frx_vertex = frx_inverseViewMatrix * frx_vertex;
      frx_vertexNormal = frx_normalModelMatrix * frx_vertexNormal;
  }
  
  frx_distance = length(frx_vertex.xyz);

  #ifdef SHADOW_MAP_PRESENT
    v_shadowViewPos = frx_shadowViewMatrix * frx_vertex;
  #endif

  // Create fake diffuse value
  float pointing_to_light;

  pointing_to_light = dot(frx_vertexNormal, frx_skyLightVector);

  v_skyDiffuse = max(0.0, pointing_to_light);
  
  pointing_to_light = (min(1.0, dot(frx_vertexNormal, vec3(-0.5, 1.0, 1.0)))) * 0.5 + 0.5;
  v_diffuse = 0.3 + 0.7 * pointing_to_light;

	v_heldCosIn = cos(frx_heldLightInnerRadius * 0.5);
	v_heldCosOut = cos(max(frx_heldLightOuterRadius * 0.5, frx_heldLightInnerRadius * 0.5));
}
