// analysis: the cheaper approximate doesn't really improve performance by much
// analysis: removing tonemap doesn't really improve performance by much
// conclusion: keep tonemap

// #define CHEAP_TONEMAP
// #define NO_TONEMAP

#ifdef NO_TONEMAP

vec3 tonemap(vec3 color) {
    return clamp(color, 0.0, 1.0);
}

vec3 reverse_tonemap(vec3 color) {
    return color;
}

float getBloom(vec3 color, float emission) {
    float color_average = (color.r + color.g + color.b) * 0.333333;
    return clamp(color_average - 1.0, 0.0, 1.0);
}

#else
#ifdef CHEAP_TONEMAP

vec3 tonemap(vec3 color) {
    return clamp((color - vec3(0.4)) / (color + vec3(1.0)) + vec3(0.4), 0.0, 1.0);
}

vec3 reverse_tonemap(vec3 color) {
    return -color / (color - vec3(0.4 + 1.0));
}

float getBloom(vec3 color, float emission) {
    float color_average = (color.r + color.g + color.b) * 0.333333;
    return clamp(2.7 * max(0.0, color_average - 0.63) + emission, 0.0, 1.0);
}

#else

vec3 tonemap(vec3 color) {
    return clamp(-exp(vec3(0.69314717) - 1.386294 * color) * 0.5 + 1, 0.0, 1.0);
}

vec3 reverse_tonemap(vec3 color) {
    return (log(-2.0 * (color - vec3(1.0))) - vec3(0.69314717)) / vec3(-1.386294);
}

float getBloom(vec3 color, float emission) {
    float color_average = (color.r + color.g + color.b) * 0.333333;
    return clamp(4 * max(0.0, color_average - 0.7) + emission, 0.0, 1.0);
}

#endif
#endif